@@ -142,7 +142,7 @@ user:portray(ATrie) :-
     format('~q for ~p', [ATrie, Goal]).
 user:portray(Cont) :-
     compound(Cont),
-    Cont =.. ['$cont$', Clause, PC | Args],
+    compound_name_arguments(Cont, '$cont$', [Clause, PC | Args]),
     clause_property(Clause, file(File)),
     file_base_name(File, Base),
     clause_property(Clause, line_count(Line)),
@@ -1129,7 +1129,7 @@ current_table_gen(M:Variant, Trie) :-
     M:'$table_mode'(Variant, NonModed, _Moded).
 current_table_gen(M:Variant, Trie) :-
     '$tbl_global_variant_table'(VariantTrie),
-    trie_gen(VariantTrie, NonModed, Trie),
+    trie_gen(VariantTrie, M:NonModed, Trie),
     \+ '$tbl_table_status'(Trie, fresh), % shared tables are not destroyed
     M:'$table_mode'(Variant, NonModed, _Moded).
 
@@ -1662,6 +1662,7 @@ propagate_assert(Head) :-
     tdebug(monotonic, 'Asserted ~p', [Head]),
     (   monotonic_dyn_affects(Head, Cont, Skel, ATrie),
         tdebug(monotonic, 'Propagating dyn ~p to ~p', [Head, ATrie]),
+        '$idg_set_current'(_, ATrie),
         pdelim(Cont, Skel, ATrie),
         fail
     ;   true
@@ -1905,8 +1906,7 @@ false_path(ATrie, BottomUp) :-
 
 false_path(ATrie, [ATrie|T], Seen) :-
     \+ memberchk(ATrie, Seen),
-    '$idg_edge'(ATrie, dependent, Dep),
-    '$tbl_reeval_wait'(Dep, Status),
+    '$idg_false_edge'(ATrie, Dep, Status),
     tdebug(reeval, '    ~p has dependent ~p (~w)', [ATrie, Dep, Status]),
     (   Status == invalid
     ->  false_path(Dep, T, [ATrie|Seen])
@@ -1971,7 +1971,7 @@ reeval_node(ATrie) :-
             )
         ;   Dep = dependency(SrcSkel, true, Cont, Skel)
         ->  (   '$member'(Node, Answers),
-                trie_term(Node, SrcSkel),
+                '$tbl_node_answer'(Node, SrcSkel),
                 tdebug(monotonic, 'Propagating ~p from ~p to ~p',
                        [Skel, SrcTrie, ATrie]),
                 pdelim(Cont, Skel, ATrie),
