@@ -799,10 +799,10 @@ set_prolog_flag_unlocked(Module m, atom_t k, term_t value, int flags ARG_LD)
 	  clear(m, M_VARPREFIX);
       } else if ( k == ATOM_debug )
       { if ( val )
-	{ debugmode(DBG_ALL, NULL);
+	{ rval = debugmode(DBG_ALL, NULL);
 	} else
-	{ tracemode(FALSE, NULL);
-	  debugmode(DBG_OFF, NULL);
+	{ rval = ( tracemode(FALSE, NULL) &&
+		   debugmode(DBG_OFF, NULL) );
 	}
       } else if ( k == ATOM_debugger_show_context )
       { debugstatus.showContext = val;
@@ -827,6 +827,8 @@ set_prolog_flag_unlocked(Module m, atom_t k, term_t value, int flags ARG_LD)
 	    PushTty(Sinput, &ttytab, TTY_SAVE);
 	  }
 	}
+      } else if ( k == ATOM_debug_on_interrupt )
+      {	rval = enable_debug_on_interrupt(val);
       } else if ( k == ATOM_protect_static_code )
       { if ( val != (f->value.a == ATOM_true) && val == FALSE )
 	{ term_t ex;
@@ -838,7 +840,7 @@ set_prolog_flag_unlocked(Module m, atom_t k, term_t value, int flags ARG_LD)
 	}
       }
 					/* set the flag value */
-      if ( f->index > 0 )
+      if ( f->index > 0 && rval )
       { unsigned int mask = (unsigned int)1 << (f->index-1);
 
 	if ( val )
@@ -878,6 +880,8 @@ set_prolog_flag_unlocked(Module m, atom_t k, term_t value, int flags ARG_LD)
       { rval = setFileNameCaseHandling(a);
       } else if ( k == ATOM_autoload )
       { rval = setAutoload(a);
+      } else if ( k == ATOM_table_monotonic )
+      { rval = setMonotonicMode(a);
 #if O_XOS
       } else if ( k == ATOM_win_file_access_check )
       { rval = set_win_file_access_check(value);
@@ -1553,6 +1557,9 @@ initPrologFlags(void)
   setPrologFlag("traditional", FT_BOOL|FF_READONLY, GD->options.traditional, 0);
   setPrologFlag("unknown", FT_ATOM, "error");
   setPrologFlag("debug", FT_BOOL, FALSE, 0);
+  setPrologFlag("debug_on_interrupt", FT_BOOL,
+		truePrologFlag(PLFLAG_DEBUG_ON_INTERRUPT),
+		PLFLAG_DEBUG_ON_INTERRUPT);
   setPrologFlag("verbose", FT_ATOM|FF_KEEP, GD->options.silent ? "silent" : "normal");
   setPrologFlag("verbose_load", FT_ATOM, "silent");
   setPrologFlag("verbose_autoload", FT_BOOL, FALSE, 0);
