@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2010-2019, University of Amsterdam
+    Copyright (c)  2010-20120, University of Amsterdam
 			      CWI, Amsterdam
     All rights reserved.
 
@@ -51,10 +51,15 @@
 #define getCharsWString(s, l)	getCharsWString__LD(s, l PASS_LD)
 #define compileTermToHeap(t, f)	\
 	compileTermToHeap__LD(t, NULL, NULL, f PASS_LD)
+#define globalizeTermRef(t)	globalize_term_ref__LD(t PASS_LD)
 #define linkVal(p)		linkVal__LD(p PASS_LD)
+#define linkValG(p)		linkValG__LD(p PASS_LD)
+#define linkValNoG(p)		linkValNoG__LD(p PASS_LD)
+#define linkValI(p)		linkValI__LD(p PASS_LD)
 #define TrailAssignment(p)	TrailAssignment__LD(p PASS_LD)
 #define bindConst(p, c)		bindConst__LD(p, c PASS_LD)
-#define consPtr(p, ts)		consPtr__LD(p, ts PASS_LD)
+#define consPtrB(p, base, ts)	consPtr__LD(p, (uintptr_t)(base), (ts) PASS_LD)
+#define consPtr(p, ts)		consPtrB(p, LD->bases[(ts)&STG_MASK], (ts))
 #define allocGlobalNoShift(n)	allocGlobalNoShift__LD(n PASS_LD)
 #define getProcDefinition(proc)	getProcDefinition__LD(proc->definition PASS_LD)
 #define popTermRef()		popTermRef__LD(PASS_LD1)
@@ -127,5 +132,8 @@
 #define var_name_ptr(p, n)	var_name_ptr__LD(p, n PASS_LD)
 #define classify_exception(ex)  classify_exception__LD(ex PASS_LD)
 #define classify_exception_p(p) classify_exception_p__LD(p PASS_LD)
+#define PL_mark_string_buffers(m) PL_mark_string_buffers__LD(m PASS_LD)
+#define PL_release_string_buffers_from_mark(m) \
+				PL_release_string_buffers_from_mark__LD(m PASS_LD)
 
 #endif /*PL_LDPASS_H_INCLUDED*/
